Attribute VB_Name = "Gif2PB"
'=====================================================
' Descrizione.....: File GIF -> PictureBox.Picture
' Nome dei Files..: Gif2PB.bas, InfoPr.frm, InfoPr.frx
' Data............: 2/10/1999
' Versione........: 1.0 a 32 bits
' Sistema.........: Visual Basic 6.0 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'=====================================================
'
'   Gif2PB.bas: Programma per leggere Files GIF
'   (anche ad immagini multiple) su PictureBox.
'   Da usare insieme a  frmInfoPr (InfoPr.frm).
'
'   Function:
'     GifToPictureBox File_GIF$, Dst As PictureBox, _
'     , [AutoSize As Boolean], [PixPix As Boolean] _
'     , [AIT$], [AAC$], [CLabel$], [BitsPerPixel&], [Interlaced As Boolean] _
'     , [ScreenW&], [ScreenH&], [ImageW&], [ImageH&], [NImages&]
'
'       File_GIF$:      File, completo di path [e con estensione .gif].
'       Dst:            Picture Box su cui si vuole riportare l' immagine GIF.
'                       Deve sempre essere Dst.ScaleMode = vbPixels
'       AutoSize:       True  -> il PictureBox Dst viene ridimensionato su
'                                ScreenW e ScreenH del file Gif (default).
'                                In questo caso Dst.Width e Dst.Height devono
'                                essere in [Pixels].
'                       False -> Dst mantiene le dimensioni originali e le
'                                immagini possono risultare tagliate.
'       PixPix:         True  -> l' immagine viene scritta sul PictureBox
'                                pixel per pixel (default).
'                       False -> viene costruita una struttura BMP copiata,
'                                successivamente, sul PictureBox.
'       AIT$:           Ritorna l' "Application Identifier".
'       AAC$:           Ritorna l' "Application Authentication Code".
'       CLabel$:        Ritorna la "Comment Label".
'       BitsPerPixel:   Ritorna il N. di [bits per pixel] del Gif.
'       Interlaced:     Ritorna l' ordine delle righe.
'                        False -> ordine sequenziale.
'                        True  -> ordine inter-allacciato.
'       ScreenW:        Ritorna la larghezza e
'       ScreenH:        l' altezza dello Schermo [pixels].
'       ImageW:         Ritorna la larghezza e
'       ImageH:         l' altezza dell' Immagine [pixels].
'       NImages:        N. di immagini contenute nel file Gif.
'
'       La funzione ritorna 0 se la lettura e' stato completata
'       con successo, altrimenti ritorna il N. di Errore.
'
'   Note:
'   (1) Il blocco "Plain Text Extension", previsto dallo standard
'       GIF89a, e' tuttora gestito da pochissimi decoders.
'       Attualmente, della decina e piu' che ho provato (VisualBasic 6.0,
'       Netscape, Internet Explorer, Photo Shop, etc...), solo due
'       (VesaView e FreeLance) hanno saputo riconoscerlo ed anche con
'       qualche limitazione.  L' ho comunque incluso per completezza.
'   (2) "The Graphics Interchange Format(c) is the Copyright property
'       of CompuServe Incorporated. GIF(sm) is a Service Mark property
'       of CompuServe Incorporated."
'   (3) L' algoritmo di compressione LZW e' brevettato da Unisys.
'       L' Unisys attualmente richiede che chiunque usi questo
'       algoritmo, per scopi commerciali, sia in possesso di una
'       loro licenza.
'
'   Per maggiori informazioni fare riferimento al file Gif89m.txt.
'
Option Explicit
'
Private GlobalColTab As Boolean ' Esiste una "Global Color Table".
Private LocalColTab As Boolean  ' Esiste una "Local Color Table".
Private GColorRes As Byte       ' "Color Resolution".
Private GSortFlag As Boolean    ' La "Global Color Table" e' in ordine.
Private BitsperPixel&           ' Vedi sopra.
Private GCT() As RGBQUAD_Type   ' "Global Color Table".
Private LCT() As RGBQUAD_Type   ' "Local Color Table".
Private ACT() As RGBQUAD_Type   ' "Active Color Table".
Private GNCol&                  ' N. di colori nella "Global Color Table".
Private LNCol&                  ' N. di colori nella "Local Color Table".
Private RBC As Boolean          ' "Restore to Back Color".
Private DelayTime!              ' Delay Time [s].
'
Private BCIx As Byte            ' "Back Color Index".
Private TCix&                   ' "Transparent Color Index".
Private PixelAspectR As Byte    ' "Pixel Aspect Ratio".
'
Private Code_Size&              ' "Code Size" [bits].
Private Code_Size1&             ' "Code Size + 1" corrente [bits].
Private NCode&                  ' N. Max corrente di Codici.
Private C_C&                    ' "Clear Code".
Private C_C1&                   ' "Clear Code + 1".
'
Private Di_Sq$()                ' Dizionario delle Sequenze.
Private DSize_I&                ' Posizione e codice iniziale nel
                                ' Dizionario delle Sequenze.
Private DSize&                  ' Posizione corrente di scrittura e
                                ' prossimo codice, nel Dizionario
                                ' delle Sequenze.
Private Const MaxDSize& = 4095  ' N. max. di codici nel Dizionario
                                ' delle Sequenze.
'
Private BufCod$                 ' Buffer dei Codici.
Private BufDB() As Byte         ' Buffer di uscita per la lettura
                                ' dei "Data Blocks" e suo
Private IB&                     ' puntatore al byte da leggere.
'
Private FG As Integer           ' # del File GIF da copiare.
'
Private XBMP&                   ' Larghezza dell' Immagine BMP [bytes].
Private IL%                     ' Bordo sinistro dell' Immagine [pixel].
Private IT%                     ' Bordo superiore dell' Immagine [pixel].
Private IW%                     ' Larghezza dell' Immagine [pixel].
Private IH%                     ' Altezza dell' Immagine [pixel].
Private SW%                     ' Larghezza dello Schermo [pixel].
Private SH%                     ' Altezza dello Schermo [pixel].
Private PicBox As PictureBox
'
Private lpBits&, hMem&          ' Variabili per l' Image
Private bi As BITMAPINFO_Type   ' del Picture Box.
Private MaxlpBits&              '
'
Private Const ISC As Byte = &H2C    ' "," -> "Image separator character".
Private Const EI As Byte = &H21     ' "!" -> "Extension Introducer".
Private Const GCL As Byte = &HF9    ' "Graphic Control Label".
Private Const PTL As Byte = &H1     ' "Plain Text Label".
Private Const AEL As Byte = &HFF    ' "Application Extension Label".
Private Const Cl As Byte = &HFE     ' "Comment Label".
Private Const GT As Byte = &H3B     ' ";" -> "GIF Trailer".
'
'
Private Type BITMAPINFOHEADER_Type
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type
'
Private Type RGBQUAD_Type
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type
'
Private Type BITMAPINFO_Type
    BitmapInfoHeader As BITMAPINFOHEADER_Type
    bmiColors(0 To 255) As RGBQUAD_Type
End Type
'
' Graphics related API:
Private Declare Function StretchDIBits Lib "gdi32" (ByVal hDC As Long _
, ByVal DestX As Long, ByVal DestY As Long, ByVal wDestWidth As Long _
, ByVal wDestHeight As Long, ByVal SrcX As Long, ByVal SrcY As Long _
, ByVal wSrcWidth As Long, ByVal wSrcHeight As Long, ByVal lpBits& _
, BitsInfo As BITMAPINFO_Type, ByVal wUsage As Long, ByVal dwRop&) As Long
'
'
' Memory allocation related API:
Private Declare Function GlobalAlloc Lib "kernel32" _
(ByVal wFlags As Long, ByVal lMem&) As Long
Private Declare Function GlobalLock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalUnlock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalFree Lib "kernel32" _
(ByVal hMem As Long) As Long
'
Private Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" _
(pDest As Any, pSource As Any, ByVal dwLength As Long)
'
Private Const BI_RGB& = 0
Private Const SRCCOPY& = &HCC0020
'
Private Const DIB_RGB_COLORS& = 0
Private Const DIB_PAL_COLORS& = 1
'
Private Const GMEM_MOVEABLE& = 2
Private Const GMEM_ZEROINIT& = &H40
'
' Desaware API:
'Private Declare Sub agCopyData Lib "apigid32.dll" _
(source As Any, dest As Any, ByVal nCount&)
Private Function Ceil(ByVal X As Double) As Double
'
'   Funzione di arrotondamento, per numeri reali,
'   all' intero uguale o immediatamente superiore:
'
    If X = Int(X) Then
        Ceil = X
    ElseIf X > 0# Then
        Ceil = Int(X) + 1#
    ElseIf X < 0# Then
        Ceil = Fix(X)
    End If
'
'
'
End Function


Private Function ScanAlign(ByVal pwidth As Long) As Long
'
'   Ritorna un multiplo di 4 uguale o immediatamente
'   superiore a pwidth&:
'
    ScanAlign = (pwidth + 3) And &HFFFFFFFC
'
'
'
End Function
Private Sub Agg_DiS(ByVal A$)
'
'   Inserisce la stringa A$ nel dizionario delle Sequenze.
'
    If DSize = MaxDSize Then Exit Sub
    ReDim Preserve Di_Sq$(DSize_I To DSize)
'
    Di_Sq$(DSize) = A$
    DSize = DSize + 1
'
    If DSize + 1 > NCode Then
        Code_Size1 = Code_Size1 + 1
        NCode = 2 ^ Code_Size1
    End If
'
'
'
End Sub

Private Function Dec2Bin$(ByVal ND&, ByVal L&)
'
'   Converte il n. decimale positivo ND& in stringa binaria di L& [bits].
'   (0 <= ND& <= 2147483647, L& < 32).
'   Deve essere ND& < 2 ^ L&: nessun controllo viene effettuato.
'   Volendo si puo' calcolare la lunghezza minima L& con la formula:
'     L = CLng(Ceil(Log(ND + 1) / Log(2))) ' con 0 < ND < 2147483647.
'
    Dim SB$, D&, I&
'
    SB$ = ""
    D = 2 ^ (L - 1)
    For I = 1 To L
        If ND - D >= 0 Then
            SB$ = SB$ & "1"
            ND = ND - D
        Else
            SB$ = SB$ & "0"
        End If
        D = D / 2
    Next I
'
    Dec2Bin$ = SB$
'
'
'
End Function

Private Sub Canc_DiS()
'
'   Re-Inizia il Dizionario delle Sequenze:
'
    Code_Size1 = Code_Size + 1      ' "Code Size + 1" [Bits].
    NCode = 2 ^ Code_Size1          ' N. Max corrente di codici.
    DSize = DSize_I                 ' Posizione corrente nel
    ReDim Di_Sq(DSize_I To DSize)   ' Dizionario delle sequenze.
'
'
'
End Sub

Private Function Seq$(ByVal Code&)
'
'   Ritorna la Sequenza Seq$ corrispondente al N. di Codice Code&.
'   Ritorna una stringa vuota se il codice Code non e' trovato:
'
    Seq$ = ""
    If Code < C_C Then  ' Cerca nel dizionario delle Radici:
        Seq$ = Chr$(Code)
    Else                ' Cerca nel dizionario delle Sequenze:
        'If Code <= UBound(Di_Sq$) Then
        If Code < DSize Then
            Seq$ = Di_Sq$(Code)
        End If
    End If
'
'
'
End Function

Private Function Bin2Dec(ByVal SB$, ByVal L&) As Long
'
'   Converte gli L& [bits] piu' significativi della stringa
'   binaria SB$ in n. decimale positivo (L& < 32).
'   Se L > Len(SB$) e' come se ci fossero L - Len(SB$) "0"
'   a destra di SB$:
'
    Dim D&, V&, I&
'
    D = 0
    V = 2 ^ (L - 1)
    For I = 1 To L
        If Mid$(SB$, I, 1) = "1" Then
            D = D + V
        End If
        V = V / 2
    Next I
'
    Bin2Dec = D
'
'
'
End Function

Private Function LZWDecode$(ByVal Code&, ByVal ILZW&)
'
'   Decompressione LZW:
'
    Dim BC$
    Static BS$
'
    If ILZW > 1 Then
        If Seq$(Code) <> "" Then
            BC$ = Left$(Seq$(Code), 1)
        Else
            BC$ = Left$(BS$, 1)
        End If
        Agg_DiS (BS$ & BC$)
    End If
'
    BS$ = Seq$(Code)
    LZWDecode = BS$
'
'
'
End Function

Private Function InC() As Long
'
'   Scompatta i Codici di lunghezza Code_Size1& [bits]
'   ed ritorna un codice per volta:
'
    Dim B&, C&
'
    Do While Len(BufCod$) < Code_Size1
        B = InB
        If B < 0 Then Exit Do
        BufCod$ = Dec2Bin$(B, 8) & BufCod$
    Loop
'
    If B >= 0 Then
        C = Bin2Dec(Right$(BufCod$, Code_Size1), Code_Size1)
        BufCod$ = Left$(BufCod$, Len(BufCod$) - Code_Size1)
    Else
        C = Bin2Dec(BufCod$, Len(BufCod$))
    End If
    InC = C
'
'
'
End Function

Private Function InB() As Long
'
'   Legge i bytes del "Data Stream" a pacchetti <= di 255 [Bytes]
'   e ritorna un byte per volta:
'
    If IB = BufDB(0) Then
        ' Legge i pacchetti dal File:
        Get #FG, , BufDB(0)
        For IB = 1 To BufDB(0)
            Get #FG, , BufDB(IB)
        Next IB
        IB = 0
    End If
'
    If BufDB(0) > 0 Then
        IB = IB + 1
        InB = BufDB(IB)
    Else
        InB = -1
    End If
'
'
'
End Function

Private Sub Leggi_ImD(Interlaced As Boolean)
'
'   Legge dal File la parte descrittiva del Gif:
'   IL% = Image Left.
'   IT% = Image Top.
'   IW% = Image Width.
'   IH% = Image Height.
'
    Dim B As Byte
'
    Get #FG, , IL
    Get #FG, , IT
    Get #FG, , IW
    Get #FG, , IH
'
    Get #FG, , B        ' -> "Packed Fields"
                        ' -> |M|I|S|RR|pixel|
    LocalColTab = B And &H80
    Interlaced = B And &H40
    If LocalColTab Then BitsperPixel = CLng((B And &H7) + 1)
'
'
'
End Sub

Private Function Leggi_Int() As Boolean
'
'   Legge dal File la parte iniziale del Gif:
'   SW% = Screen Width.
'   SH% = Screen Height.
'
    Dim A As String * 6, B As Byte
'
    ' "Header":
    Get #FG, 1, A
    If Not ((A = "GIF87a") Or (A = "GIF89a")) Then
        Leggi_Int = False
        Exit Function
    End If
'
    ' "Logical Screen Descriptor":
    Get #FG, , SW
    Get #FG, , SH
'
    ' "|x|xxx|x|xxx|" -> |M|cr|S|pixel|
    Get #FG, , B    ' -> "Packed Fields"
    GlobalColTab = B And &H80
    GColorRes = ((B And &H70) / &H10) + 1
    GSortFlag = B And &H8
    BitsperPixel = (B And &H7) + 1
'
    Get #FG, , BCIx         ' "Background Color Index".
'
    Get #FG, , PixelAspectR ' "Pixel Aspect Ratio".
'
    Leggi_Int = True
'
'
'
End Function

Private Sub Leggi_GCT()
'
'   Legge dal File la tabella globale dei colori:
'
    Dim C&
'
    For C = 0 To GNCol - 1
        Get #FG, , GCT(C).rgbRed
        Get #FG, , GCT(C).rgbGreen
        Get #FG, , GCT(C).rgbBlue
    Next C
'
'
'
End Sub


Private Sub Leggi_CEx(CLabel$)
'
'   Legge il "Comment Extension":
'
    Dim C&
'
    CLabel$ = ""
    Do
        C = InB
        If C < 0 Then Exit Do
        CLabel$ = CLabel$ + Chr$(C)
    Loop
'
'
'
End Sub

Private Sub Ord_ScanRows(ScanRows&(), ByVal Interlaced As Boolean)
'
'   Prepara il vettore contenente l' ordine delle
'   righe per ricostruire l' Immagine dal file GIF.
'
    Dim Y&, Passo&, Riga&
    ReDim ScanRows(1 To IH)
'
    If Interlaced Then      ' GIF in ordine inter_allacciato:
        Y = 0
        For Passo = 1 To 4
            Select Case Passo
                Case 1
                For Riga = 1 To IH Step 8
                    Y = Y + 1
                    ScanRows(Y) = Riga - 1
                Next Riga
'
                Case 2
                For Riga = 5 To IH Step 8
                    Y = Y + 1
                    ScanRows(Y) = Riga - 1
                Next Riga
'
                Case 3
                For Riga = 3 To IH Step 4
                    Y = Y + 1
                    ScanRows(Y) = Riga - 1
                Next Riga
'
                Case 4
                For Riga = 2 To IH Step 2
                    Y = Y + 1
                    ScanRows(Y) = Riga - 1
                 Next Riga
            End Select
        Next Passo
'
    Else                    ' GIF in ordine sequenziale:
        For Y = 1 To IH
            ScanRows(Y) = Y - 1
        Next Y
    End If
'
'
'
End Sub

Private Sub Leggi_PTE(MSin&, MSup&, TGW&, TGH&, CCW&, CCH& _
    , BCol&, FCol&, Ptext$)
'
'   Legge il "Plain Text Extension":
'
    Dim M$, C&, I As Integer, B As Byte
'
    Get #FG, , B    ' "Block Size" sempre di 12 [bytes].
'
    Get #FG, , I    ' "Text Grid Left Position".
    MSin = CLng(I)
'
    Get #FG, , I    ' "Text Grid Top Position".
    MSup = CLng(I)
'
    Get #FG, , I    ' "Text Grid Width".
    TGW = CLng(I)
'
    Get #FG, , I    ' "Text Grid Height".
    TGH = CLng(I)
'
    Get #FG, , B    ' "Character Cell Width".
    CCW = CLng(B)
'
    Get #FG, , B    ' "Character Cell Height".
    CCH = CLng(B)
'
    Get #FG, , B    ' "Text Foreground Color Index".
    FCol = CLng(B)
'
    Get #FG, , B    ' "Text Background Color Index".
    BCol = CLng(B)
'
    Do
        C = InB
        If C < 0 Then Exit Do
        Ptext$ = Ptext$ + Chr$(C)
    Loop
'
'
'
End Sub

Private Sub Leggi_GCE()
'
'   Legge il "Graphic Control Extension":
'
    Dim I As Integer, B As Byte, TCF As Boolean
'
    Get #FG, , B    ' "Block Size" sempre di 4 [bytes].
'
    Get #FG, , B    ' -> "Packed Fields" -> "|RRR|DDD|U|T|"
    TCF = B And &H1 ' "Transparent Color Flag".
    RBC = B And &H8 ' "Restore to Back Color".
'
    Get #FG, , I    ' "Delay Time".
    DelayTime = CSng(I) / 100
'
    Get #FG, , B    ' "Transparent Color Index".
    TCix = IIf(TCF, CLng(B), -1)
'
    Get #FG, , B    ' "Block Terminator"
'
'
'
End Sub


Public Function GifToPictureBox(ByVal File_GIF$, ByVal Dst As PictureBox _
    , Optional ByVal AutoSize As Boolean = True _
    , Optional ByVal PixPix As Boolean = True _
    , Optional AIT$ = "", Optional AAC$ = "", Optional CLabel$ = "" _
    , Optional BxP&, Optional Interlaced As Boolean _
    , Optional ScreenW&, Optional ScreenH& _
    , Optional ImageW&, Optional ImageH& _
    , Optional NImages&) As Long
'
'   Questa routine usa sempre (se PixPix = False) un BitMap
'   a 256 colori.
'
    Dim I&, ILZW&, C$, X&, XIP&, Y&, YIP&, ByC&, ByI&
    Dim InBuf As Byte, Code&, CIx&, PBCIx&, R1&
    Dim TempoI!, ScanRows&()
    Dim MSin&, MSup&, TGW&, TGH&, CCW&, CCH&, BCol&, FCol&, Ptext$
'
    On Error GoTo GifToPictureBox_ERR
'
    frmInfoPr.Apri File_GIF$, "Legge File GIF -> PictureBox", vbNormal
    Set PicBox = Dst
'
    TCix = -1       ' "Transparent Color Index".
    NImages = 0
    ReDim BufDB(0 To 255)
'
    FG = FreeFile
    Open File_GIF$ For Binary As #FG
'
    ' Legge l' "Header" ed il "Logical Screen Descriptor":
    If Not Leggi_Int Then Err.Raise 17
    ScreenW = CLng(SW)
    ScreenH = CLng(SH)
    If AutoSize Then
        Dst.Width = ScreenW
        Dst.Height = ScreenH
    End If
    If GlobalColTab Then
        BxP = BitsperPixel
        GNCol = 2 ^ BitsperPixel
        ReDim GCT(0 To GNCol - 1)
        Leggi_GCT       ' Legge la "Global Color Table".
        ACT() = GCT()   ' Tabella Colori Corrente = "Global Color Table".
    End If
'
    Do  ' Esplora il "Data Stream":
        Get #FG, , InBuf
        Select Case InBuf
            Case EI             ' "Extension Introducer"
            Get #FG, , InBuf
            Select Case InBuf
                Case GCL        ' "Graphic Control Label".
                Leggi_GCE
'
                Case PTL        ' "Plain Text Label".
                Leggi_PTE MSin, MSup, TGW, TGH, CCW, CCH _
                , BCol, FCol, Ptext$
                If Ptext$ <> "" Then Scrivi_PTE MSin, MSup _
                , TGW, TGH, CCW, CCH, BCol, FCol, Ptext$, TCix
'
                Case AEL        ' "Application Extension Label".
                Leggi_AEL AIT$, AAC$
'
                Case Cl         ' "Comment Label".
                Leggi_CEx CLabel$
            End Select
'
            Case ISC            ' "Image Descriptor".
            Leggi_ImD Interlaced
            ImageW = CLng(IW)
            ImageH = CLng(IH)
            If LocalColTab Then
                BxP = BitsperPixel
                LNCol = 2 ^ BitsperPixel
                ReDim LCT(0 To LNCol - 1)
                Leggi_LCT       ' Legge la "Local Color Table".
                ACT() = LCT()   ' Tabella Colori Corrente = "Local Color Table".
            Else
                ACT() = GCT()
            End If
'
            PBCIx = Cerca_Col(&H1000000)        ' Inizializza la funzione Cerca_Col e
            PBCIx = Cerca_Col(PicBox.BackColor) ' trova l' indice del PicBox.BackColor.
            NImages = NImages + 1
            Ord_ScanRows ScanRows&(), Interlaced
'
            Get #FG, , InBuf            ' Legge il "Code Size".
            Code_Size = CLng(InBuf)
            C_C = 2 ^ Code_Size         ' Codice "Clear Code".
            C_C1 = C_C + 1              ' Codice "Clear Code + 1".
            DSize_I = C_C + 2           ' Posizione iniziale del Dizionario.
'
            Canc_DiS        ' Azzera il dizionario ed
            BufCod$ = ""    ' imposta le condizioni
            IB = 0          ' iniziali.
'
            ILZW = 0
            X = 0
            Y = 1
            XIP = X + IL
            YIP = ScanRows(Y) + IT
            If Not PixPix Then
                If Not PreparaBMP(8) Then Err.Raise 17
                ByI = (IH - ScanRows(Y) - 1) ' Byte d' inizio della
                ByI = ByI * XBMP + lpBits    ' riga corrente.
                ByC = 0                      ' Byte corrente nella riga.
            End If
'
            Do  ' Esplora la "Table Based Image Data":
                If EOF(FG) Then Err.Raise 62
                Code = InC
                If Code = C_C Then
                    ' Ripristina i parametri della decompressione:
                    Canc_DiS
                    ILZW = 0
                ElseIf Code = C_C1 Then
                    ' Termina la lettura dell' "Image Data Stream":
                    Code = InC
                    If Code <> 0 Then Err.Raise 513
                    Code = -1
                ElseIf Code >= 0 Then
                    ' Decodifica il codice:
                    ILZW = ILZW + 1
                    C$ = LZWDecode(Code, ILZW)
'
                    For I = 1 To Len(C$)
                        CIx = CLng(Asc(Mid$(C$, I, 1)))
                        XIP = X + IL
                        If PixPix Then
                            If CIx <> TCix Then
                                SetPixel XIP, YIP, CIx
                            ElseIf RBC Then
                                SetPixel XIP, YIP, PBCIx
                            End If
                        Else
                            If CIx = TCix Then
                                If RBC Then
                                    CIx = PBCIx
                                Else
                                    CIx = Cerca_Col(PicBox.Point(XIP, YIP))
                                End If
                            End If
                            Agg_BMP ByI, ByC, CByte(CIx)
                            ByC = ByC + 1
                        End If
'
                        X = X + 1
                        If X >= IW And Y < IH Then
                            X = 0
                            Y = Y + 1
                            YIP = ScanRows(Y) + IT
'
                            If Not PixPix Then
                                ByI = (IH - ScanRows(Y) - 1)
                                ByI = ByI * XBMP + lpBits
                                ByC = 0
                            End If
'
                            frmInfoPr.Info CSng(Y) / CSng(IH)
                            DoEvents
                            If frmInfoPr.Termina Then Err.Raise 18
                        End If
                    Next I
                End If
            Loop While Code >= 0
            If Not PixPix Then
                Scrivi_BMP
            End If
'
            TempoI = Timer + DelayTime
            Do
                DoEvents
                If frmInfoPr.Termina Then Err.Raise 18
            Loop While Timer < TempoI
        End Select
    Loop While InBuf <> GT  ' GT -> ";" -> "GIF Trailer".
'
'
GifToPictureBox_ERR:
    GifToPictureBox = Err.Number
'
    If FG > 0 Then Close #FG
    If hMem <> 0 Then
        R1 = GlobalUnlock(hMem)
        hMem = GlobalFree(hMem)
    End If
    Unload frmInfoPr
'
'
'
End Function
Public Function GifToPictureBox_V(ByVal File_GIF$, ByVal Dst As PictureBox _
    , Optional ByVal AutoSize As Boolean = True _
    , Optional ByVal PixPix As Boolean = True _
    , Optional AIT$ = "", Optional AAC$ = "", Optional CLabel$ = "" _
    , Optional BxP&, Optional Interlaced As Boolean _
    , Optional ScreenW&, Optional ScreenH& _
    , Optional ImageW&, Optional ImageH& _
    , Optional NImages&) As Long
'
'   Questa routine usa (se PixPix = False) un BitMap a numero di
'   colori variabili. Usa meno memoria ma e' piu' lenta.
'
    Dim I&, ILZW&, C$, X&, XIP&, Y&, YIP&, PBCIx&
    Dim ByC&, ByI&, CBin$, BxP_BMP&
    Dim InBuf As Byte, Code&, CIx&, R1&
    Dim TempoI!, ScanRows&()
    Dim MSin&, MSup&, TGW&, TGH&, CCW&, CCH&, BCol&, FCol&, Ptext$
'
    On Error GoTo GifToPictureBox_V_ERR
'
    frmInfoPr.Apri File_GIF$, "Legge File GIF -> PictureBox", vbNormal
    Set PicBox = Dst
'
    TCix = -1       ' "Transparent Color Index".
    NImages = 0
    ReDim BufDB(0 To 255)
'
    FG = FreeFile
    Open File_GIF$ For Binary As #FG
'
    ' Legge l' "Header" ed il "Logical Screen Descriptor":
    If Not Leggi_Int Then Err.Raise 17
    ScreenW = CLng(SW)
    ScreenH = CLng(SH)
    If AutoSize Then
        Dst.Width = ScreenW
        Dst.Height = ScreenH
    End If
    If GlobalColTab Then
        BxP = BitsperPixel
        GNCol = 2 ^ BitsperPixel
        ReDim GCT(0 To GNCol - 1)
        Leggi_GCT       ' Legge la "Global Color Table".
        ACT() = GCT()   ' Tabella Colori Corrente = "Global Color Table".
    End If
'
    Do  ' Esplora il "Data Stream":
        Get #FG, , InBuf
        Select Case InBuf
            Case EI             ' "Extension Introducer"
            Get #FG, , InBuf
            Select Case InBuf
                Case GCL        ' "Graphic Control Label".
                Leggi_GCE
'
                Case PTL        ' "Plain Text Label".
                Leggi_PTE MSin, MSup, TGW, TGH, CCW, CCH _
                , BCol, FCol, Ptext$
                If Ptext$ <> "" Then Scrivi_PTE MSin, MSup _
                , TGW, TGH, CCW, CCH, BCol, FCol, Ptext$, TCix
'
                Case AEL        ' "Application Extension Label".
                Leggi_AEL AIT$, AAC$
'
                Case Cl         ' "Comment Label".
                Leggi_CEx CLabel$
            End Select
'
            Case ISC            ' "Image Descriptor".
            Leggi_ImD Interlaced
            ImageW = CLng(IW)
            ImageH = CLng(IH)
            If LocalColTab Then
                BxP = BitsperPixel
                LNCol = 2 ^ BitsperPixel
                ReDim LCT(0 To LNCol - 1)
                Leggi_LCT       ' Legge la "Local Color Table".
                ACT() = LCT()   ' Tabella Colori Corrente = "Local Color Table".
            Else
                ACT() = GCT()
            End If
'
            PBCIx = Cerca_Col(&H1000000)        ' Inizializza la funzione Cerca_Col e
            PBCIx = Cerca_Col(PicBox.BackColor) ' trova l' indice del PicBox.BackColor.
            NImages = NImages + 1
            Ord_ScanRows ScanRows&(), Interlaced
'
            Get #FG, , InBuf            ' Legge il "Code Size".
            Code_Size = CLng(InBuf)
            C_C = 2 ^ Code_Size         ' Codice "Clear Code".
            C_C1 = C_C + 1              ' Codice "Clear Code + 1".
            DSize_I = C_C + 2           ' Posizione iniziale del Dizionario.
'
            Canc_DiS        ' Azzera il dizionario ed
            BufCod$ = ""    ' imposta le condizioni
            IB = 0          ' iniziali.
            CBin$ = ""      ' <--
'
            ILZW = 0
            X = 0
            Y = 1
            XIP = X + IL
            YIP = ScanRows(Y) + IT
            If Not PixPix Then
                BxP_BMP = BitsperPixel
                If BxP_BMP > 1 And BxP_BMP < 4 Then BxP_BMP = 4
                If BxP_BMP > 4 And BxP_BMP < 8 Then BxP_BMP = 8
                If Not PreparaBMP(BxP_BMP) Then Err.Raise 17
                ByI = (IH - ScanRows(Y) - 1) ' Byte d' inizio della
                ByI = ByI * XBMP + lpBits    ' riga corrente.
                ByC = 0                      ' Byte corrente nella riga.
            End If
'
            Do  ' Esplora la "Table Based Image Data":
                If EOF(FG) Then Err.Raise 62
                Code = InC
                If Code = C_C Then
                    ' Ripristina i parametri della decompressione:
                    Canc_DiS
                    ILZW = 0
                ElseIf Code = C_C1 Then
                    ' Termina la lettura dell' "Image Data Stream":
                    Code = InC
                    If Code <> 0 Then Err.Raise 513
                    Code = -1
                ElseIf Code >= 0 Then
                    ' Decodifica il codice:
                    ILZW = ILZW + 1
                    C$ = LZWDecode(Code, ILZW)
'
                    For I = 1 To Len(C$)
                        CIx = CLng(Asc(Mid$(C$, I, 1)))
                        XIP = X + IL
                        If PixPix Then
                            If CIx <> TCix Then
                                SetPixel XIP, YIP, CIx
                            ElseIf RBC Then
                                SetPixel XIP, YIP, PBCIx
                            End If
                        Else
                            If CIx = TCix Then
                                If RBC Then
                                    CIx = PBCIx
                                Else
                                    CIx = Cerca_Col(PicBox.Point(XIP, YIP))
                                End If
                            End If
'
                            CBin$ = CBin$ & Dec2Bin$(CIx, BxP_BMP)
'
                            If Len(CBin$) = 8 Then
                                Agg_BMP ByI, ByC, CByte(Bin2Dec(CBin$, 8))
                                CBin$ = ""
                                ByC = ByC + 1
                            End If
                        End If
'
                        X = X + 1
                        If X >= IW And Y < IH Then
                            X = 0
                            Y = Y + 1
                            YIP = ScanRows(Y) + IT
'
                            If Not PixPix Then
                                If CBin$ <> "" Then
                                    Agg_BMP ByI, ByC, CByte(Bin2Dec(CBin$, 8))
                                End If
                                ByI = (IH - ScanRows(Y) - 1)
                                ByI = ByI * XBMP + lpBits
                                ByC = 0
                                CBin$ = ""
                            End If
'
                            frmInfoPr.Info CSng(Y) / CSng(IH)
                            DoEvents
                            If frmInfoPr.Termina Then Err.Raise 18
                        End If
                    Next I
                End If
            Loop While Code >= 0
            If Not PixPix Then
                If CBin$ <> "" Then
                    Agg_BMP ByI, ByC, CByte(Bin2Dec(CBin$, 8))
                End If
                Scrivi_BMP
            End If
'
            TempoI = Timer + DelayTime
            Do
                DoEvents
                If frmInfoPr.Termina Then Err.Raise 18
            Loop While Timer < TempoI
        End Select
    Loop While InBuf <> GT  ' GT -> ";" -> "GIF Trailer".
'
'
GifToPictureBox_V_ERR:
    GifToPictureBox_V = Err.Number
'
    If FG > 0 Then Close #FG
    If hMem <> 0 Then
        R1 = GlobalUnlock(hMem)
        hMem = GlobalFree(hMem)
    End If
    Unload frmInfoPr
'
'
'
End Function
Private Function Cerca_Col(ByVal VCol&) As Long
'
'   Cerca, nella tabella attiva dei colori, il colore
'   che piu' si avvicina a VCol e ne ritorna l' indice:
'
    Dim R&, G&, B&, C&, DQ&, DisQ&
    Static CC&, VCol_O&
'
    If VCol_O <> VCol Then
        VCol_O = VCol
'
        R = VCol And &HFF           ' Componente Rossa.
        G = (VCol \ &H100) And &HFF ' Componente Verde.
        B = VCol \ &H10000          ' Componente Blu.
'
        DisQ = 10000000
        For C = 0 To UBound(ACT)
            DQ = (R - ACT(C).rgbRed) * (R - ACT(C).rgbRed)
            DQ = DQ + (G - ACT(C).rgbGreen) * (G - ACT(C).rgbGreen)
            DQ = DQ + (B - ACT(C).rgbBlue) * (B - ACT(C).rgbBlue)
            If DisQ > DQ Then
                CC = C
                DisQ = DQ
            End If
        Next C
    End If
'
    Cerca_Col = CC
'
'
'
End Function

Private Sub SetPixel(ByVal X&, ByVal Y&, ByVal CInx&)
'
'   Disegna sul PictureBox, alle coordinate X e Y, il punto
'   di colore corrispondente all' indice CInx nella tavola
'   dei colori:
'
    If CInx <= UBound(ACT) Then
        PicBox.PSet (X, Y) _
        , RGB(ACT(CInx).rgbRed, ACT(CInx).rgbGreen, ACT(CInx).rgbBlue)
    End If
'
'
'
End Sub

Private Sub Leggi_LCT()
'
'   Legge dal File la tabella locale dei colori:
'
    Dim C&
'
    For C = 0 To LNCol - 1
        Get #FG, , LCT(C).rgbRed
        Get #FG, , LCT(C).rgbGreen
        Get #FG, , LCT(C).rgbBlue
    Next C
'
'
'
End Sub

Private Sub Scrivi_PTE(ByVal MSin&, ByVal MSup& _
    , ByVal TGW&, ByVal TGH&, ByVal CCW&, ByVal CCH& _
    , ByVal BCol&, ByVal FCol&, ByVal Ptext$, ByVal TCix&)
'
'   Scrive sul PictureBox il "Plain Text Extension":
'
    Dim BC&, FC&, NCol&, NRig&
    Dim C&, R&, A&
'
    If BCol <> TCix Then
        BC = RGB(GCT(BCol).rgbRed, GCT(BCol).rgbGreen, GCT(BCol).rgbBlue)
        PicBox.Line (MSin, MSup)-(TGW + MSin, TGH + MSup), BC, BF
    End If
'
    If FCol <> TCix Then
        FC = RGB(GCT(FCol).rgbRed, GCT(FCol).rgbGreen, GCT(FCol).rgbBlue)
        NCol = TGW / CCW
        NRig = TGH / CCH
        PicBox.Font = "Terminal"
        PicBox.FontSize = PicBox.ScaleY(CCH, vbPixels, vbPoints)
        PicBox.ForeColor = FC
'
        A = 1
        For R = 1 To NRig
            PicBox.CurrentY = MSup + CCH * (R - 1)
            For C = 1 To NCol
                PicBox.CurrentX = MSin + CCW * (C - 1)
                PicBox.Print Mid$(Ptext$, A, 1);
                A = A + 1
                If A > Len(Ptext$) Then Exit Sub
            Next C
        Next R
    End If
'
'
'
End Sub

Private Sub Leggi_AEL(AIT$, AAC$)
'
'   Legge la "Application Extension":
'
    Dim AI As String * 8, AA As String * 3, C&, B As Byte
'
    Get #FG, , B    ' "Block Size" sempre di 11 [bytes].
'
    Get #FG, , AI    ' "Application Identifier".
    AIT$ = Trim$(AI)
'
    Get #FG, , AA    ' "Application Autentication Code".
    AAC$ = AA
'
    Dim Ptext$
    Do
        C = InB
        If C < 0 Then Exit Do
        Ptext$ = Ptext$ + Chr$(C)
    Loop
'
'
'
End Sub


Private Function PreparaBMP(ByVal BxP_BMP&) As Boolean
'
'   Prepara il bi ed il blocco di memoria hMem ad indirizzo lpBits
'   di una struttura BMP a numero di colori 2 ^ BxP_BMP (2, 16 , 256):
'
    Dim BytesNeeded&, C&
'
    On Error GoTo PreparaBMP_ERR
'
    ' Fill the BITMAPINFO for the desired DIB:
    bi.BitmapInfoHeader.biSize = Len(bi.BitmapInfoHeader)
    bi.BitmapInfoHeader.biWidth = IW
    bi.BitmapInfoHeader.biHeight = IH
    bi.BitmapInfoHeader.biPlanes = 1    '1 ColorPlane.
    bi.BitmapInfoHeader.biBitCount = CInt(BxP_BMP)
    bi.BitmapInfoHeader.biCompression = BI_RGB
'
    ' Fill the Color Table:
    For C = 0 To UBound(ACT)
        bi.bmiColors(C).rgbRed = ACT(C).rgbRed
        bi.bmiColors(C).rgbGreen = ACT(C).rgbGreen
        bi.bmiColors(C).rgbBlue = ACT(C).rgbBlue
    Next C
'
    ' Calculate the number of bytes needed to store the image:
    XBMP = ScanAlign(IW * BxP_BMP / 8)  ' La XBMP& deve essere
                                        ' un multiplo di 4.
    BytesNeeded = XBMP * IH
'
    ' Allocate a buffer to hold the bitmap bits:
    hMem = GlobalAlloc(GMEM_MOVEABLE + GMEM_ZEROINIT, BytesNeeded)
'
    If hMem <> 0 Then
        lpBits = GlobalLock(hMem)
        MaxlpBits = lpBits + BytesNeeded
    Else
        Err.Raise 513
    End If
'
'
PreparaBMP_ERR:
    PreparaBMP = (Err = 0)
'
'
'
End Function

Public Sub Scrivi_BMP()
'
'   Riporta sul PictureBox l' immagine contenuta
'   in bi ed hMem ad indirizzo lpBits:
'
    Dim R1&
'
    R1 = StretchDIBits(PicBox.hDC, CLng(IL), CLng(IT), CLng(IW), CLng(IH) _
    , 0, 0, bi.BitmapInfoHeader.biWidth, bi.BitmapInfoHeader.biHeight _
    , ByVal lpBits, bi, DIB_RGB_COLORS, SRCCOPY)
    PicBox.Refresh
'
    If hMem <> 0 Then
        ' Libera il blocco di memoria
        ' appena usato:
        R1 = GlobalUnlock(hMem)
        hMem = GlobalFree(hMem)
    End If
'
'
'
End Sub

Private Function Agg_BMP(ByVal ByI&, ByVal ByC&, ByVal B As Byte) As Boolean
'
'   Aggiorna l' hMem di una struttura BMP
'   all' indirizzo ByI + ByC:
'
    Dim ByteAddr&
'
    ByteAddr = ByI + ByC
    If ByteAddr < MaxlpBits Then
        'agCopyData B, ByVal ByteAddr, 1
        MoveMemory ByVal ByteAddr, B, 1
        Agg_BMP = True
    Else
        Agg_BMP = False
    End If
'
'
'
End Function
