Attribute VB_Name = "PB2Gif_M"
'==========================================================
' Descrizione.....: Array di PictureBox.Picture -> File GIF
' Nome dei Files..: PB2Gif_M.bas, InfoPr.frm, InfoPr.frx
' Data............: 7/6/2000
' Versione........: 1.0 a 32 bits
' Sistema.........: Visual Basic 6.0
'                   sotto Windows NT 4.0 (SP 3).
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'==========================================================
'
'   PB2Gif_M.bas: Programma per salvare l' Image di
'   PictureBoxes multiple su file GIF.
'   Da usare insieme a frmInfoPr (InfoPr.frm).
'
'   Function:
'     PictureBoxesToGif Src() As PictureBox, File_GIF$, [BackTras As Boolean] _
'     , [BitsPerPixel&], [LocalCT As Boolean], [Interlaced As Boolean] _
'     , [TCol], [MSin&], [MSup&], [NCln&], [NRig&], [LCln&], [ARig&] _
'     , [BCol&], [FCol&], [PText$]
'
'       Src():          Array di PictureBoxes di cui si vuole salvare l' Image su
'       File_GIF$:      File, completo di path [e con estensione .gif].
'       BackTras:       Blocca l' immagine di Src come BackGround BitMap.
'                        Per Windows NT:
'                        True  -> copia sul file GIF anche i grafici generati
'                                 al "Run Time" con i metodi Line, Circle,
'                                 Print, etc.. (default).
'                        False -> copia sul file GIF solo l' Image di
'                                 BackGround.
'                        Per Windows 98: deve essere sempre True (default).
'       BitsPerPixel:   N. di [bits per pixel]. Opzionale e puo' essere solo:
'                        1 -> l' immagine viene salvata in Bianco e Nero.
'                        4 -> l' immagine viene salvata a 16 colori.
'                        8 -> l' immagine viene salvata a 256 colori (default).
'       LocalCT:        Richiede la generazione delle "Local Color Tables".
'                       Opzionale (default = False).
'                       Per Windows NT: se BackTras = False e' consigliabile
'                       l' uso delle "Local Color Tables".
'       Interlaced:     Ordine delle righe. Opzionale:
'                        False -> ordine sequenziale (default).
'                        True  -> ordine inter-allacciato. Permette, durante
'                                 un caricamento lento (e.g. pagine Web), di
'                                 vedere il contenuto generale dell' immagine
'                                 prima che tutti i dati siano stati
'                                 elaborati/trasmessi.
'       TDelay:         Intervallo tra le elaborazioni di immagini
'                       successive in 1/100 di [s].
'       TCol:           Colore che si vuole rendere trasparente.
'                       Se il parametro e' assente o ha valore Null (default)
'                       nessun colore viene reso come trasparente.
'
'       Parametri del "Plain Text Extension":
'       MSin&, MSup&:   Margini sinistro e superiore [pixels].
'       NCln&, NRig&:   Numero di colonne e di righe.
'       LCln&, ARig&:   Larghezza ed altezza di colonne e di righe [pixels].
'       BCol&, FCol&:   BackColor e ForeColor del testo.
'       PText$:         Testo da inserire sull' immagine.
'
'       La funzione ritorna 0 se il trasferimento e' stato completato
'       con successo, altrimenti ritorna il N. di Errore.
'
'   Note:
'   (1) E' stata riportata, a solo scopo illustrativo e non
'       utilizzata, la struttura standard di compressione LZW con
'       doppio dizionario (delle radici e delle sequenze): in effetti,
'       l' uso di variabili di tipo String, permette di semplificare
'       l' algoritmo e di usare un solo dizionario (delle sequenze).
'       Il dizionario delle sequenze e' stato definito come struttura
'       di una String e della sua lunghezza: in questo modo diventa,
'       mediamente, piu' veloce la ricerca del N. di codice associato
'       ad una certa sequenza (nella routine Code).
'   (2) Il blocco "Plain Text Extension", previsto dallo standard
'       GIF89a, e' tuttora gestito da pochissimi decoders.
'       Attualmente, della decina e piu' che ho provato (VisualBasic 6.0,
'       Netscape, Internet Explorer, Photo Shop, etc...), solo due
'       (VesaView e FreeLance) hanno saputo riconoscerlo ed anche con
'       qualche limitazione.  L' ho comunque incluso per completezza.
'   (3) Del blocco "Graphic Control Extension"  ho utilizzato solo
'       i controlli di Delay Time e Transparency Index.
'   (4) "The Graphics Interchange Format(c) is the Copyright property
'       of CompuServe Incorporated. GIF(sm) is a Service Mark property
'       of CompuServe Incorporated."
'   (5) L' algoritmo di compressione LZW e' brevettato da Unisys.
'       L' Unisys attualmente richiede che chiunque usi questo
'       algoritmo, per scopi commerciali, sia in possesso di una
'       loro licenza.
'
'       Per maggiori informazioni fare riferimento al file Gif89m.txt.
'
Option Explicit
'
Private BitsperPixel&           ' Vedi sopra.
Private NCol&                   ' N. di colori in Tabella.
Private Code_Size&              ' "Code Size" [bits].
Private Code_Size1&             ' "Code Size + 1" corrente [bits].
Private NCode&                  ' N. Max corrente di Codici.
Private C_C&                    ' "Clear Code".
Private C_C1&                   ' "Clear Code + 1".
'
'Private Di_Ra() As String * 1   ' Dizionario delle Radici.
Private Type Sequenza_Type
    L As Long               ' Lunghezza della Sequenza.
    S As String             ' Stringa della Sequenza.
End Type
Private Di_Sq() As Sequenza_Type ' Dizionario delle Sequenze.
Private DSize_I&                ' Posizione e codice iniziale nel
                                ' Dizionario delle Sequenze.
Private DSize&                  ' Posizione corrente di scrittura e
                                ' prossimo codice, nel Dizionario
                                ' delle Sequenze.
Private Const MaxDSize& = 511   ' Posizione finale e codice Max.
                                ' del Dizionario delle Sequenze:
                                ' MAXDSIZE& = 2^x - 1 (x > BitsPerPixel).
                                ' Aumentandolo si riducono le dimensioni
                                ' del file GIF ma si aumenta il tempo di
                                ' elaborazione e l' uso di memoria. Deve
                                ' comunque essere MaxDSize <= 4095.
'
Private BufCod$                 ' Buffer dei Codici.
Private BufDB(0 To 255) As Byte ' Buffer di uscita per la scrittura
                                ' dei "Data Blocks" e suo
Private IB&                     ' puntatore al byte da scrivere.
'
Private FG As Integer           ' # del File GIF temporaneo.
'
Private XBMP&()                 ' Larghezza dell' Immagine BMP [bytes].
Private IW%()                   ' Larghezza dell' Immagine [pixels].
Private IH%()                   ' Altezza dell' Immagine [pixels].
Private SW%                     ' Larghezza dello Schermo [pixels].
Private SH%                     ' Altezza dello Schermo [pixels].
'
Private lpBits&(), hMem&()      ' Variabili per l' Image
Private bi() As BITMAPINFO_Type ' del Picture Box.
'
'
Private Type BITMAPINFOHEADER_Type
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type
'
Private Type RGBQUAD_Type
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type
'
Private Type BITMAPINFO_Type
    BitmapInfoHeader As BITMAPINFOHEADER_Type
    bmiColors(0 To 255) As RGBQUAD_Type
End Type
'
Private Type RECT_Type
    Left  As Long
    Top  As Long
    Right  As Long
    Bottom  As Long
End Type
'
' DC related API
Private Declare Function CreateCompatibleDC Lib "gdi32" _
(ByVal hDC As Long) As Long
Private Declare Function GetWindowDC Lib "user32" _
(ByVal hWnd As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" _
(ByVal hDC As Long) As Long
'
' Graphics related API
Private Declare Function GetDIBits Lib "gdi32" _
(ByVal hDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long _
, ByVal nNumScans As Long, ByVal lpBits As Long _
, BITMAPINFO As BITMAPINFO_Type, ByVal wUsage As Long) As Long
'
' General attribute related API
Declare Function GetClientRect Lib "user32" _
(ByVal hWnd As Long, lpRect As RECT_Type) As Long
'
' Memory allocation related API
Private Declare Function GlobalAlloc Lib "kernel32" _
(ByVal wFlags As Long, ByVal lMem&) As Long
Private Declare Function GlobalLock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalUnlock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalFree Lib "kernel32" _
(ByVal hMem As Long) As Long
'
Private Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" _
(pDest As Any, pSource As Any, ByVal dwLength As Long)
'
Private Const BI_RGB& = 0
'
Private Const DIB_RGB_COLORS& = 0
Private Const DIB_PAL_COLORS& = 1
'
Private Const GMEM_MOVEABLE& = 2
'
' Temporary files API:
Private Const MAX_PATH& = 260
Private Declare Function GetTempPath Lib "kernel32" _
Alias "GetTempPathA" (ByVal nBufferLength As Long _
, ByVal lpBuffer As String) As Long
Private Declare Function GetTempFileName Lib "kernel32" _
Alias "GetTempFileNameA" (ByVal lpszPath As String _
, ByVal lpPrefixString As String, ByVal wUnique As Long _
, ByVal lpTempFileName As String) As Long

Private Function Ceil(ByVal X As Double) As Double
'
'   Funzione di arrotondamento, per numeri reali,
'   all' intero uguale o immediatamente superiore:
'
    If X = Int(X) Then
        Ceil = X
    ElseIf X > 0# Then
        Ceil = Int(X) + 1#
    ElseIf X < 0# Then
        Ceil = Fix(X)
    End If
'
'
'
End Function

Private Function Legge_PictureBox(ByVal I&, ByVal PBSrc As PictureBox) As Boolean
'
'
    Dim hDC_PB&, hDC_Mem&, BytesNeeded&
    Dim R1&, PB_Width&, PB_Height&, Rect As RECT_Type
'
    On Error GoTo Legge_PictureBox_ERR
'
    ' Create a temporary memory DC and select into it
    ' the background picture of the PBSrc Picture Box:
    hDC_PB = GetWindowDC(PBSrc.hWnd)
    hDC_Mem = CreateCompatibleDC(hDC_PB)
'
    R1 = GetClientRect(PBSrc.hWnd, Rect)
    PB_Width = Abs(Rect.Right - Rect.Left)
    PB_Height = Abs(Rect.Bottom - Rect.Top)
'
    ' Fill the BITMAPINFO for the desired DIB:
    bi(I).BitmapInfoHeader.biSize = Len(bi(I).BitmapInfoHeader)
    bi(I).BitmapInfoHeader.biWidth = PB_Width
    bi(I).BitmapInfoHeader.biHeight = PB_Height
    bi(I).BitmapInfoHeader.biPlanes = 1    'ColorPlanes
    bi(I).BitmapInfoHeader.biBitCount = BitsperPixel
    bi(I).BitmapInfoHeader.biCompression = BI_RGB
'
    ' Calculate the number of bytes needed to store the image:
    BytesNeeded = ScanAlign(PB_Width * BitsperPixel / 8) * PB_Height
'
    ' Allocate a buffer to hold the bitmap bits:
    hMem(I) = GlobalAlloc(GMEM_MOVEABLE, BytesNeeded)
'
    If hMem(I) <> 0 Then
        lpBits(I) = GlobalLock(hMem(I))
        ' Get the bits that make up the image:
        R1 = GetDIBits(hDC_Mem, PBSrc.Picture, 0, PB_Height _
        , lpBits(I), bi(I), DIB_RGB_COLORS)
    Else
        Err.Raise vbObjectError + 3
    End If
'
'
Legge_PictureBox_ERR:
    Legge_PictureBox = (Err = 0)
'
    R1 = DeleteDC(hDC_Mem)
'
'
'
End Function
Private Function TempFile$(Optional ByVal I3C$ = "TMP" _
    , Optional ByRef TempPath As String)
'
'   Ritorna il Nome di un File temporaneo [da cancellare
'   esplicitamente dopo l' uso].
'   Con I3C$ si possono specificare fino a tre caratteri
'   iniziali per l' identificazione del file:
'
    Dim R1&, lpBuffer$
    Dim TempFileName As String
'
    R1 = GetTempPath(0, "")
    lpBuffer$ = Space(R1)
    R1 = GetTempPath(R1, lpBuffer$)
    TempPath = Left$(lpBuffer$, R1)
'
    TempFileName = Space(MAX_PATH)
    R1 = GetTempFileName(TempPath, I3C$, 0, TempFileName)
    TempFile$ = Trim$(TempFileName)
'
'
'
End Function

Private Function ScanAlign(ByVal pwidth As Long) As Long
'
'   Ritorna un multiplo di 4 uguale o immediatamente
'   superiore a pwidth&:
'
    ScanAlign = (pwidth + 3) And &HFFFFFFFC
'
'
'
End Function
Private Sub Agg_DiS(ByVal A$)
'
'   Inserisce la stringa A$ nel dizionario delle Sequenze.
'
    Dim L&, I&
'
    L = Len(A$)
    If L > 1 Then
        Di_Sq(DSize).L = L
        Di_Sq(DSize).S = A$
'
        If DSize > NCode Then
            Code_Size1 = Code_Size1 + 1
            NCode = 2 ^ Code_Size1
        End If
'
        DSize = DSize + 1
    End If
'
'
'
End Sub

Private Function Dec2Bin$(ByVal ND&, ByVal L&)
'
'   Converte il n. decimale positivo ND& in stringa binaria di L& [bits].
'   (0 <= ND& <= 2147483647, L& < 32).
'   Deve essere ND& < 2 ^ L&: nessun controllo viene effettuato.
'   Volendo si puo' calcolare la lunghezza minima L& con la formula:
'     L = CLng(Ceil(Log(ND + 1) / Log(2))) ' con 0 < ND < 2147483647.
'
    Dim SB$, D&, I&
'
    SB$ = ""
    D = 2 ^ (L - 1)
    For I = 1 To L
        If ND - D >= 0 Then
            SB$ = SB$ & "1"
            ND = ND - D
        Else
            SB$ = SB$ & "0"
        End If
        D = D / 2
    Next I
'
    Dec2Bin$ = SB$
'
'
'
End Function

Private Sub Canc_DiS()
'
'   Re-Inizia il Dizionario delle Sequenze:
'
    Code_Size1 = Code_Size + 1
    NCode = 2 ^ Code_Size1
    DSize = DSize_I
    ReDim Di_Sq(DSize_I To MaxDSize) 'As Dictionary
'
'
'
End Sub

Private Function Code(ByVal A$) As Long
'
'   Cerca la stringa A$ (radice o sequenza) nei dizionari e ne
'   ritorna il N. di Codice.
'   Ritorna -1 se la stringa A$ non e' trovata:
'
    Dim L&, I&
'
    L = Len(A$)
    If L = 1 Then   ' Cerca nel dizionario delle Radici:
        'For I = 0 To NCol - 1
        '    If A$ = Di_Ra(I) Then
        '        Code = I
        '        Exit Function
        '    End If
        'Next I
        Code = Asc(A$)
        Exit Function
    Else            ' Cerca nel dizionario delle Sequenze:
        For I = DSize_I To DSize - 1
            If Di_Sq(I).L = L Then
                If A$ = Di_Sq(I).S Then
                    Code = I
                    Exit Function
                End If
            End If
        Next I
    End If
'
    Code = -1  ' La stringa A$ non e' presente nei dizionari.
'
'
'
End Function

Private Function Bin2Dec(ByVal SB$, ByVal L&) As Long
'
'   Converte gli L& [bits] piu' significativi della stringa
'   binaria SB$ in n. decimale positivo (L& < 32):
'
    Dim D&, V&, I&
'
    D = 0
    V = 2 ^ (L - 1)
    For I = 1 To L
        If Mid$(SB$, I, 1) = "1" Then
            D = D + V
        End If
        V = V / 2
    Next I
'
    Bin2Dec = D
'
'
'
End Function

Private Function FileExists(ByVal FileN$) As Boolean
'
'   Verifica l' esistenza del file FileN$:
'
    Dim res&
'
    On Error Resume Next
'
    res = FileLen(FileN$)
    FileExists = (Err = 0)
'
'
'
End Function


Private Function Legge_BMP(ByVal I&, ByVal WPText&, ByVal HPText&) As Boolean
'
'
    On Error GoTo Legge_BMP_ERR
'
    ' Legge i Parametri del DIB:
    IW(I) = CInt(bi(I).BitmapInfoHeader.biWidth)
    IH(I) = CInt(bi(I).BitmapInfoHeader.biHeight)
    XBMP(I) = ScanAlign(IW(I) * BitsperPixel / 8)   ' La XBMP& deve essere
                                                    ' un multiplo di 4.
    NCol = 2 ^ BitsperPixel
'
    SW = MAX(SW, IW(I), WPText)   ' Screen Width >= Image Width.
    SH = MAX(SH, IH(I), HPText)   ' Screen Height >= Image Height.
'
'
Legge_BMP_ERR:
    Legge_BMP = (Err = 0)
'
'
'
End Function
Private Function MAX(ParamArray D() As Variant) As Variant
'
'   Implementa la funzione MAX(D1, D2, ...DN) del FORTRAN:
'
    Dim J&, J1&, J2&, DMax As Variant
'
    J1 = LBound(D)
    J2 = UBound(D)
    DMax = D(J1)
    For J = J1 + 1 To J2
        If DMax < D(J) Then DMax = D(J)
    Next J
'
    MAX = DMax
'
'
'
End Function

Private Function LZWEncode(ByVal C$, ByVal ILZW&) As Long
'
'   Compressione LZW:
'
    Static W$
'
    If ILZW = 1 Then
        W$ = C$
        'Agg_DiS W$  ' E' sempre una radice.
        LZWEncode = -1
'
    ElseIf ILZW = -1 Then
        LZWEncode = Code(W$)
'
    Else
        If Code(W$ & C$) >= 0 Then
            W$ = W$ & C$
            LZWEncode = -1
        Else
            LZWEncode = Code(W$)
            Agg_DiS W$ & C$
            W$ = C$
        End If
    End If
'
'
'
End Function

Private Sub OutB(ByVal C&)
'
'   Formatta i Codici in lunghezza Code_Size1& [bits]
'   ed emette un Byte per volta:
'
    Dim SB$, B&
'
    SB$ = Dec2Bin$(C, Code_Size1)
    BufCod$ = SB$ & BufCod$
    Do While Len(BufCod$) >= 8
        B = Bin2Dec(Right$(BufCod$, 8), 8)
        PacB B
        BufCod$ = Left$(BufCod$, Len(BufCod$) - 8)
    Loop
'
    If C = C_C1 Then    ' Termina la scrittura del "Data Stream":
        B = Bin2Dec(BufCod$, Len(BufCod$))
        PacB B
        PacB -1
    End If
'
'
'
End Sub

Private Sub PacB(ByVal B&)
'
'   Compone i bytes di uscita in pacchetti di 255 [Bytes]
'   e li scrive sul file FileGifTemp$:
'
    Dim I&
'
    If B >= 0 Then
        IB = IB + 1
        BufDB(IB) = CByte(B)
        If IB = 255 Then
            GoSub ScriviPac
        End If
    Else
        ' Emette, se e' il caso, l' ultimo
        ' pacchetto > 0 e <= di 255 [Bytes]:
        If IB > 0 Then GoSub ScriviPac
        ' Scrive il "Block Terminator":
        GoSub ScriviPac
    End If
    Exit Sub
'
'
ScriviPac:      ' Scrive i pacchetti sul File:
    BufDB(0) = CByte(IB)
    For I = 0 To IB
        Put #FG, , BufDB(I)
    Next I
    IB = 0
    Return
'
'
'
End Sub

Private Sub Scrivi_GTr()
'
'   Scrive sul File la parte terminale del Gif:
'
    Dim B As Byte
'
    B = &H3B        ' ";"
    Put #FG, , B    ' -> "GIF Trailer".
'
'
'
End Sub

Private Sub Scrivi_ImD(ByVal I&, ByVal IL%, ByVal IT% _
    , ByVal LocalCT As Boolean, ByVal Interlaced As Boolean)
'
'   Scrive sul File la parte descrittiva del Gif:
'   IL% = Image Left.
'   IT% = Image Top.
'   IW% = Image Width.
'   IH% = Image Height.
'
    Dim B As Byte
'
    B = &H2C            ' "," ->
    Put #FG, , B        ' "Image separator character".
'
    Put #FG, , IL
    Put #FG, , IT
    Put #FG, , IW(I)
    Put #FG, , IH(I)
'
    If Interlaced Then
        B = &H40        ' "|0|1|0|00|xxx|" -> |M|I|S|RR|pixel|
    Else
        B = &H0         ' "|0|0|0|00|xxx|" -> |M|I|S|RR|pixel|
    End If
'
    If LocalCT Then
        B = B Or &H80   ' "|1|x|0|00|xxx|" -> |M|I|S|RR|pixel|
        B = B Or CByte(BitsperPixel - 1)
    End If
'
    Put #FG, , B        ' -> "Packed Fields".
'
'
'
End Sub

Private Sub Scrivi_Int(ByVal GlobalCT As Boolean)
'
'   Scrive sul File la parte iniziale del Gif:
'   SW% = Screen Width.
'   SH% = Screen Height.
'
    Dim A$, B As Byte
'
    ' "Header":
    A$ = "GIF89a"
    Put #FG, 1, A$
'
    ' "Logical Screen Descriptor":
    Put #FG, , SW
    Put #FG, , SH
'
    If GlobalCT Then
        B = &HF0    ' "|1|111|0|xxx|" -> |M|cr|S|pixel|
    Else
        B = &H70    ' "|0|111|0|xxx|" -> |M|cr|S|pixel|
    End If
    B = B Or CByte(BitsperPixel - 1)
    Put #FG, , B    ' -> "Packed Fields"
'
    B = &H0         ' "Background Color Index".
    Put #FG, , B
'
    B = &H0         ' "Pixel Aspect Ratio".
    Put #FG, , B
'
'
'
End Sub

Private Sub Scrivi_GCT(ByVal I&)
'
'   Scrive sul File la tabella globale dei colori:
'
    Dim C&
'
    For C = 0 To NCol - 1
        Put #FG, , bi(I).bmiColors(C).rgbRed
        Put #FG, , bi(I).bmiColors(C).rgbGreen
        Put #FG, , bi(I).bmiColors(C).rgbBlue
    Next C
'
'
'
End Sub


Public Function PictureBoxesToGif(Src() As PictureBox, ByVal File_GIF$ _
    , Optional ByVal BackTras As Boolean = True _
    , Optional ByVal BxP& = 8 _
    , Optional ByVal LocalCT As Boolean = False _
    , Optional ByVal Interlaced As Boolean = False _
    , Optional ByVal TDelay& = 0 _
    , Optional ByVal TCol As Variant = Null _
    , Optional ByVal MSin& = 0, Optional ByVal MSup& = 0 _
    , Optional ByVal NCln& = 0, Optional ByVal NRig& = 0 _
    , Optional ByVal LCln& = 8, Optional ByVal ARig& = 16 _
    , Optional ByVal BCol& = vbWhite, Optional ByVal FCol& = vbBlack _
    , Optional ByVal Ptext$ = "") As Long
'
'
    Dim I&, Is1&, Is2&
    Dim WPText&, HPText&
    Dim GlobalCT As Boolean
    Dim ILZW&, XIP&, YIP&, ByI&, ByC&, PixN&
    Dim ByteAddr&, InBuf As Byte, ScanRows&()
    Dim C$, A&, R1&, FileN$, FileGifTemp$
    Dim Passi&, CBin$, Ps&, P&, TFlag&
'
    On Error GoTo PictureBoxesToGif_ERR
'
    GlobalCT = (Ptext$ <> "") Or (Not LocalCT)
'
    frmInfoPr.Apri File_GIF$, "Salva PictureBox -> File GIF", vbNormal
    Is1 = LBound(Src)
    Is2 = UBound(Src)
    ReDim lpBits(Is1 To Is2), hMem(Is1 To Is2)
    ReDim bi(Is1 To Is2) As BITMAPINFO_Type
    ReDim IW(Is1 To Is2), IH(Is1 To Is2), XBMP(Is1 To Is2)
'
    If BackTras Then
        ' Blocca l' immagine di Src(I) come Background BitMap.
        ' Funziona anche con scheda video impostata a
        ' profondita' di colore = 8 [bits per pixel]:
        FileN$ = TempFile$
        For I = Is1 To Is2
            SavePicture Src(I).Image, FileN$
            Src(I).Picture = LoadPicture("")
            Src(I).Picture = LoadPicture(FileN$)
        Next I
        If FileExists(FileN$) Then Kill FileN$
    End If
'
    BitsperPixel = BxP
    Passi = Switch(BitsperPixel = 1, 8, BitsperPixel = 4, 2 _
    , BitsperPixel = 8, 1)
'
    ' Prepara la "Transparent Color Flag".
    If VarType(TCol) = vbNull Then
        TFlag = 0
        TCol = 0
    Else
        TFlag = 1
    End If
'
    SW = 0
    SH = 0
    If Ptext$ <> "" Then
        WPText = MSin + NCln * LCln
        HPText = MSup + NRig * ARig
    End If
    For I = Is1 To Is2
        If Not Legge_PictureBox(I, Src(I)) Then Err.Raise vbObjectError + 1
        If Not Legge_BMP(I, WPText, HPText) Then Err.Raise vbObjectError + 1
    Next I
'
    'PalettePrint
'
    FG = FreeFile
    FileGifTemp$ = TempFile$("GIF")
    Open FileGifTemp$ For Binary As #FG
'
    Code_Size = IIf(BitsperPixel = 1, 2, BitsperPixel)
    C_C = 2 ^ Code_Size         ' Codice "Clear Code".
    C_C1 = C_C + 1              ' Codice "Clear Code + 1".
    DSize_I = C_C + 2           ' Posizione iniziale del Dizionario.
'
    ReDim Di_Sq(DSize_I To MaxDSize)  ' Dizionario delle Sequenze.
'
'
    Scrivi_Int GlobalCT         ' Scrive sul File l' intestazione
                                ' del GIF ("Header" + "Logical
                                ' Screen Descriptor").
    If GlobalCT Then
        Scrivi_GCT Is1          ' Scrive sul File la "Global Color Table"
    End If                      ' con la palette della prima immagine.
'
    For I = Is1 To Is2
        Code_Size1 = Code_Size + 1  ' "Code Size + 1" [Bits].
        NCode = 2 ^ Code_Size1      ' N. Max corrente di Codici.
        DSize = DSize_I             ' Posizione corrente nel Dizionario.
        BufCod$ = ""                ' Azzera il Buffer dei codici
        IB = 0                      ' ed il puntatore nel Buffer dei
                                    ' "Data Blocks".
'
        ' Prepara il vettore dell' ordine delle righe:
        Ord_ScanRows I, ScanRows(), Interlaced
        Scrivi_GCE I, TDelay, TFlag, TCol       ' Scrive il "Graphic Control Extension".
'
        Scrivi_ImD I, 0, 0, LocalCT, Interlaced ' Scrive sul File l' "Image Descriptor".
        If LocalCT Then
            Scrivi_GCT (I)                      ' Scrive sul File la "Local Color Table".
        End If
        Scrivi_BID                              ' Scrive sul File il primo
                                                ' byte dell' "Image Data".
'
'       Inizia la Compressione:
        ILZW = 1                    ' N. di chiamata dell' algoritmo LZW.
        OutB C_C                    ' Inizia il Blocco Dati Compressi
                                    ' (facoltativo ma consigliato).
'
        For YIP = 1 To IH(I)
            ByI = ScanRows(YIP) * XBMP(I) + lpBits(I)   ' Byte d' inizio della
                                                        ' riga corrente.
            ByC = 0                                     ' Byte corrente nella riga.
            Ps = 0                                      ' Passo corrente nel Byte.
            For XIP = 1 To IW(I)
                If Ps = 0 Then
                    ByteAddr = ByI + ByC
                    MoveMemory InBuf, ByVal ByteAddr, 1
                    CBin$ = Dec2Bin$(InBuf, 8)
                    Ps = Passi
                    ByC = ByC + 1
                End If
'
                P = 9 - BitsperPixel * Ps
                C$ = Chr$(Bin2Dec(Mid$(CBin$, P, BitsperPixel), BitsperPixel))
'
                A = LZWEncode(C$, ILZW)
                If A >= 0 Then
                    OutB A
                End If
                ILZW = ILZW + 1
'
                If DSize >= MaxDSize Then
                    A = LZWEncode("", -1) ' Termina il Processo corrente
                    OutB A                ' di Compressione.
                    OutB C_C              ' Re-Inizia il Blocco Dati Compressi.
                    Canc_DiS              ' Azzera il Dizionario delle Sequenze.
                    ILZW = 1
                End If
'
                Ps = Ps - 1
            Next XIP
'
            DoEvents
            If frmInfoPr.Termina Then Err.Raise 18
            frmInfoPr.Info CSng(YIP) / CSng(IH(I))  ' Informa sulla percentuale
                                                    ' di lavoro fatto.
        Next YIP
'
'
        A = LZWEncode("", -1)   ' Termina il Processo di
        OutB A                  ' Compressione.
        OutB C_C1               ' Codice "Clear Code + 1": termina il
                                ' "Data Stream".
        Scrivi_CEx Src(I).Name  ' Scrive la "Comment Extension".
    Next I
'
    If Ptext$ <> "" Then        ' Scrive, sull' immagine, il "Plain Text".
        Scrivi_PTE MSin, MSup _
        , NCln, NRig, LCln, ARig _
        , BCol, FCol, Ptext$
    End If
'
    Scrivi_GTr              ' Scrive sul File il codice di "GIF
                            ' Trailer".
'
    Close #FG: FG = 0       ' Chiude il File per poterlo copiare.
    FileCopy FileGifTemp$, File_GIF$  ' Salva sul File richiesto.
'
'
PictureBoxesToGif_ERR:
    PictureBoxesToGif = Err.Number
'
    If FG > 0 Then Close #FG
    If FileExists(FileGifTemp$) Then Kill FileGifTemp$
    For I = Is1 To Is2
        If hMem(I) <> 0 Then
            R1 = GlobalUnlock(hMem(I))
            hMem(I) = GlobalFree(hMem(I))
        End If
    Next I
    Unload frmInfoPr
'
'
'
End Function

Private Sub Scrivi_CEx(ByVal PicBoxName$)
'
'   Scrive il "Comment Extension":
'
    Dim M$, C&, B As Byte
'
    B = &H21        ' "!"
    Put #FG, , B    ' -> "Extension Introducer".
'
    B = &HFE        ' "Extension Function Code"
    Put #FG, , B    ' -> "Comment Label".
'
    M$ = PicBoxName$ & " "
    M$ = M$ & "F. Languasco - CopyRight 1999"
'
    For C = 1 To Len(M$)
        B = AscB(Mid$(M$, C, 1))
        PacB B
    Next C
    PacB -1
'
'
'
End Sub

Private Sub Ord_ScanRows(ByVal I&, ScanRows&(), ByVal Interlaced As Boolean)
'
'   Prepara il vettore ScanRows&() contenente l' ordine delle
'   righe dell' Immagine da copiare sul GIF.
'   L' ordine viene anche invertito, perche' il BMP usa le
'   righe dal basso all' alto ed il GIF dall' alto al basso.
'
    Dim Y&, Passo&, Riga&
    ReDim ScanRows(1 To IH(I))
'
    If Interlaced Then      ' GIF in ordine inter_allacciato:
        Y = 0
        For Passo = 1 To 4
            Select Case Passo
                Case 1
                For Riga = 1 To IH(I) Step 8
                    Y = Y + 1
                    ScanRows(Y) = IH(I) - Riga
                Next Riga
'
                Case 2
                For Riga = 5 To IH(I) Step 8
                    Y = Y + 1
                    ScanRows(Y) = IH(I) - Riga
                Next Riga
'
                Case 3
                For Riga = 3 To IH(I) Step 4
                    Y = Y + 1
                    ScanRows(Y) = IH(I) - Riga
                Next Riga
'
                Case 4
                For Riga = 2 To IH(I) Step 2
                    Y = Y + 1
                    ScanRows(Y) = IH(I) - Riga
                 Next Riga
            End Select
        Next Passo
'
    Else                    ' GIF in ordine sequenziale:
        For Y = 1 To IH(I)
            ScanRows(Y) = IH(I) - Y
        Next Y
    End If
'
'
'
End Sub

Private Sub Scrivi_PTE(ByVal MSin&, ByVal MSup& _
    , ByVal NCln&, ByVal NRig&, ByVal LCln&, ByVal ARig& _
    , ByVal BCol&, ByVal FCol&, ByVal Ptext$)
'
'   Scrive il "Plain Text Extension":
'
    Dim M$, C&, I As Integer, B As Byte
'
    B = &H21                ' "!"
    Put #FG, , B            ' -> "Extension Introducer".
'
    B = &H1                 ' "Extension Function Code"
    Put #FG, , B            ' -> "Plain Text Label".
'
    B = 12                  ' "Block Size" sempre di 12 [bytes].
    Put #FG, , B
'
    I = MSin                ' "Text Grid Left Position"
    Put #FG, , I
'
    I = MSup                ' "Text Grid Top Position"
    Put #FG, , I
'
    I = NCln * LCln         ' "Text Grid Width"
    Put #FG, , I
'
    I = NRig * ARig         ' "Text Grid Height"
    Put #FG, , I
'
    B = LCln                ' "Character Cell Width"
    Put #FG, , B
'
    B = ARig                ' "Character Cell Height"
    Put #FG, , B
'
    B = Cerca_Col(1, FCol)    ' "Text Foreground Color Index"
    Put #FG, , B
'
    B = Cerca_Col(1, BCol)    ' "Text Background Color Index"
    Put #FG, , B
'
    For C = 1 To Len(Ptext$)
        B = AscB(Mid$(Ptext$, C, 1))
        PacB B
    Next C
    PacB -1
'
'
'
End Sub

Private Sub Scrivi_GCE(ByVal I&, ByVal TDelay&, ByVal TFlag&, Optional ByVal TCol&)
'
'   Scrive il "Graphic Control Extension":
'
    Dim J As Integer, B As Byte
'
    B = &H21                ' "!"
    Put #FG, , B            ' -> "Extension Introducer".
'
    B = &HF9                ' "Extension Function Code"
    Put #FG, , B            ' -> "Graphic Control Label".
'
    B = 4                   ' "Block Size" sempre di 4 [bytes].
    Put #FG, , B
'
    'B = &H0                 ' "|000|000|0|x|" -> "|RRR|DDD|U|T|"
    'B = &H4                 ' "|000|001|0|x|" -> "|RRR|DDD|U|T|"
    B = &H8                 ' "|000|010|0|x|" -> "|RRR|DDD|U|T|"
    B = B Or CByte(TFlag)
    Put #FG, , B            ' -> "Packed Fields".
'
    J = TDelay              ' "Delay Time".
    Put #FG, , J
'
    If TFlag = 1 Then
        B = Cerca_Col(I, TCol) ' "Transparent Color Index".
    Else
        B = 0
    End If
    Put #FG, , B
'
    B = 0                   ' "Block Terminator".
    Put #FG, , B
'
'
'
End Sub

Private Function Cerca_Col(ByVal I&, ByVal VCol&) As Byte
'
'   Cerca, nella tabella dei colori di Src(I), il colore
'   che piu' si avvicina a VCol e ne ritorna l' indice:
'
    Dim R&, G&, B&, C&, CC&, DQ&, DisQ&
'
    R = VCol And &HFF           ' Componente Rossa.
    G = (VCol \ &H100) And &HFF ' Componente Verde.
    B = VCol \ &H10000          ' Componente Blu.
'
    DisQ = 10000000
    For C = 0 To NCol - 1
        DQ = (R - CLng(bi(I).bmiColors(C).rgbRed)) ^ 2
        DQ = DQ + (G - CLng(bi(I).bmiColors(C).rgbGreen)) ^ 2
        DQ = DQ + (B - CLng(bi(I).bmiColors(C).rgbBlue)) ^ 2
        If DisQ > DQ Then
            CC = C
            DisQ = DQ
        End If
    Next C
'
    Cerca_Col = CByte(CC)
'
'
'
End Function

Public Sub Scrivi_BID()
'
'   Scrive sul file il primo byte dell' Image Data:
'
    Dim B As Byte
'
    B = CByte(Code_Size)
    Put #FG, , B
'
'
'
End Sub

Private Sub PalettePrint()
'
'   Stampa, per debug, le palettes delle immagini:
'
    Dim I&, Is1&, Is2&, C&
'
    Is1 = LBound(bi)
    Is2 = UBound(bi)
'
    For C = 0 To NCol - 1
        Debug.Print "R" & Str$(C) & ":";
        For I = Is1 To Is2
            Debug.Print bi(I).bmiColors(C).rgbRed,
        Next I
        Debug.Print
        Debug.Print "G" & Str$(C) & ":";
        For I = Is1 To Is2
            Debug.Print bi(I).bmiColors(C).rgbGreen,
        Next I
        Debug.Print
        Debug.Print "B" & Str$(C) & ":";
        For I = Is1 To Is2
            Debug.Print bi(I).bmiColors(C).rgbBlue,
        Next I
        Debug.Print
    Next C
'
'
'
End Sub

